#include <stdio.h>
#include <stdlib.h>

#include <mpi.h>

#define PRINTMTX 0

int main (int argc, char ** argv) {

  int n, i, j;
  double stime, etime;
  int size, rank, rlen;
  char myname[MPI_MAX_PROCESSOR_NAME]; 

  MPI_Init (&argc, &argv);

  MPI_Comm_size (MPI_COMM_WORLD, &size);
  
  MPI_Comm_rank (MPI_COMM_WORLD, &rank);

  MPI_Get_processor_name (myname, &rlen);

  fprintf (stdout, "I am proces %3d of %3d my name is %s (%s)\n", 
    rank, size, myname, argv[0]);

  fflush(stdout);

  if (argc != 2) {
    fprintf (stderr, "usage: %s N\n", argv[0]);
    MPI_Finalize ();
    return EXIT_FAILURE;
  } 

  if ( (n = atoi (argv[1])) < 0) {
    fprintf (stderr, "N must be gt 0\n");
    MPI_Finalize ();
    return EXIT_FAILURE;
  }

  if (size == 2) {
    double * d;
    MPI_Status status;
    MPI_Request request;

    if ( (d = (double *) malloc (sizeof (double) * n * n)) == NULL) {
      fprintf (stderr, "Memory allocation error\n");
      return EXIT_FAILURE;
    }

    if (rank == 0) {
      int flag = 0;

      srand (10);
      for (i=0; i<n; i++)
        for (j=0; j<n; j++)
          d[(i*n) + j] = 1.0e0 + (double) (1.0e0 * n * 
              (rand() / (RAND_MAX + 1.0e0)));   

      stime = MPI_Wtime ();
      MPI_Isend(d, n*n, MPI_DOUBLE, 1, 10, MPI_COMM_WORLD, &request);
      etime = MPI_Wtime ();
      fprintf (stdout, "Send (%d) time 3: %f s\n", rank, etime - stime);
      while (!flag) {
        int i, a = 0;
        
        for (i=0; i<100000; i++) 
          a++;

        MPI_Test (&request, &flag, &status);
        fprintf (stdout, "I am working...\n");
      }
    }
    else {
      stime = MPI_Wtime ();
      MPI_Recv(d, n*n, MPI_DOUBLE, 0, 10, MPI_COMM_WORLD, &status);
      etime = MPI_Wtime ();
      fprintf (stdout, "Recv (%d) time 3: %f s\n", rank, etime - stime);
    }

    free (d);
  }

  MPI_Finalize ();

  return EXIT_SUCCESS;
}
